Package: tbm
Title: Transformation Boosting Machines
Version: 0.3-11
Date: 2026-06-10
Authors@R: person("Torsten", "Hothorn", role = c("aut", "cre"),
                  email = "Torsten.Hothorn@R-project.org", comment = c(ORCID = "0000-0001-8301-0471"))
Description: Boosting the likelihood of conditional and shift transformation models as introduced in <DOI:10.1007/s11222-019-09870-4>.
Depends: mlt (>= 1.7-0), mboost (>= 2.8-2)
Imports: variables, basefun, sandwich, coneproj, methods
Suggests: TH.data (>= 1.0-9), tram (>= 1.3-0), survival, partykit,
        lattice, latticeExtra, knitr, colorspace, gamlss.data, trtf
VignetteBuilder: knitr
URL: https://codeberg.org/thothorn/tram
License: GPL-2
NeedsCompilation: no
Packaged: 2026-06-10 15:49:20 UTC; hothorn
Author: Torsten Hothorn [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8301-0471>)
Maintainer: Torsten Hothorn <Torsten.Hothorn@R-project.org>
Repository: CRAN
Date/Publication: 2026-06-10 17:10:02 UTC
Built: R 4.5.2; ; 2026-06-10 17:23:02 UTC; unix
