
This folder contains R code necessary to reproduce the empirical results
presented in Section 5 of "Transformation Boosting Machines". Simple run

  make -f Makefile.sim
  make -f Makefile.sim summary.pdf

  make -f Makefile.app
  make -f Makefile.app summary.pdf

in either sub-folder (after installing the tbm R add-on package).
